<?php
/**
 * @package RSDirectory!
 * @copyright (C) 2013-2018 www.rsjoomla.com
 * @license GPL, http://www.gnu.org/copyleft/gpl.html
 */

// No direct access.
defined('_JEXEC') or die('Restricted access');

/**
 * RSDirectory! Carousel Module Helper.
 */
abstract class RSDirectoryCarouselHelper {
	
	public static $params;
	
	protected static $featured_categories;
	
    /**
     * Get entries.
     *
     * @access public
     *
     * @static
     *
     * @param JRegistry $params
     *
     * @return mixed
     */
    public static function getEntries() {
		// Get DBO.
		$db = JFactory::getDbo();
			
		$select = array(
			$db->qn('e') . '.*',
            $db->qn('c.title', 'category_title'),
            $db->qn('c.path', 'category_path'),
            $db->qn('u.name', 'author'),
		);
			
		$query = $db->getQuery(true)
			   ->select($select)
			   ->from($db->qn('#__rsdirectory_entries', 'e'))
			   ->innerJoin($db->qn('#__categories', 'c') . ' ON ' . $db->qn('e.category_id') . ' = ' . $db->qn('c.id'))
			   ->innerJoin($db->qn('#__users', 'u') . ' ON ' . $db->qn('e.user_id') . ' = ' . $db->qn('u.id'))
			   ->where($db->qn('e.published') . ' = ' . $db->q(1))
			   ->where($db->qn('u.block') . ' = ' . $db->q(0))
			   ->where('(' . $db->qn('e.expiry_time') . ' = ' . $db->q('0000-00-00 00:00:00') . ' OR ' . $db->qn('e.expiry_time') . ' > ' . $db->q( JFactory::getDate()->toSql() ) . ')');
			   
		if (self::$params->get('show_promoted', 0)) {
			$query->where($db->qn('e.promoted').' = '.$db->q(1));
		}
			
		$categories_ids = self::getCategories();
		
		if (empty($categories_ids)) {
			$categories_ids = array(0);
		}
		
		if ($categories_ids) {
			// Add the categories condition.
			$query->where($db->qn('e.category_id') . ' IN (' . implode(',', array_unique($categories_ids)) . ')');
		}
			
		// Set the ordering criteria.
		$order_by = self::$params->get('order_by');
		$order = self::$params->get('order');
		
		
		if ($order_by == 'rating') {
			$query->order($db->qn('e.avg_rating') . ' ' . $db->escape($order) . ', ' . $db->qn('e.ratings_count') . ' ASC');
		}
		else if ($order_by == 'random') {
			$query->order(' RAND()');
		}
		else {
			$query->order( $db->qn($order_by) . ' ' . $db->escape($order));
		}
		
		$db->setQuery($query, 0, self::$params->get('max_entries'));
		$entries = $db->loadObjectList();
		
		if (self::$params->get('display_thumbs')) {
			return RSDirectoryHelper::getEntriesData($entries);
		}
		return $entries;
    }
	
	protected static function getCategories() {
		self::$featured_categories = RSDirectoryHelper::arrayInt(self::$params->get('featured_categories'));
		if (empty(self::$featured_categories) || in_array(0, self::$featured_categories)) {
			self::$featured_categories = array();
		}
		
		$options = array(
			'extension' 	=> 'com_rsdirectory',
			'table' 		=> '#__rsdirectory_entries',
			'field' 		=> 'category_id',
			'statefield' 	=> 'published',
			'countItems' 	=> false
		);
			
		// Create a new JCategories object.
		$categories = new JCategories($options);
		
		if ($root = $categories->get('root')) {
			if ($children = $root->getChildren()) {				
				return self::getIds($children);
			}
		}
		
		return false;
	}
	
	protected static function getIds($categories) {
		$ids = array();

		foreach ($categories as $category) {
			$has_children = $category->hasChildren();
			$children = null;

			// if features categories are defined search inside
			if (self::$featured_categories) {
				$children_ids = array();

				if ($has_children) {
					$children = $category->getChildren();

					foreach ($children as $child) {
						$children_ids[] = $child->id;
					}

					$intersect = array_intersect(self::$featured_categories, $children_ids);

					// if the child categories are not in the featured categories but the parent is add all the children to the ids
					if (count($intersect) == 0 && in_array($category->id, self::$featured_categories)) {
						$tmp = array_merge(self::$featured_categories, $children_ids);
						self::$featured_categories = array_unique($tmp);
					}
				}

				// if the category is in the featured categories add it to the ids
				if (in_array($category->id, self::$featured_categories)) {
					$ids[] = $category->id;
				}
			} else {
				// if no featured categories put all the ids
				$ids[] = $category->id;
			}

			// if the category has any children parse them too
			if ($has_children) {
				$ids = array_merge($ids, self::getIds((!is_null($children) ? $children : $category->getChildren())));
			}
		}

		return $ids;
	}
		
	/**
	 * Method to get a list of entries per slide.s
	 *
	 * @access public
	 *
	 * @static
	 *
	 * @param array $entries
	 * @param int $limit
	 *
	 * @return mixed
	 */
	public static function getEntriesPerSlide($entries, $limit) {
		$slides	= array();
		$count	= 0;
		$index	= 0;
			
		foreach ($entries as $entry) {
			$count++;
			
			if ($count > $limit) {
				$count = 1;
				$index++;
			}
			if (!isset($slides[$index])) {
				$slides[$index] = array();	
			}
			$slides[$index][] = $entry;
		}
		return $slides;
	}
}